import React from "react";
import ReactDOM from "react-dom";
import MyComponent from "./ToggleList";

ReactDOM.render(
  <React.StrictMode>
    <MyComponent />
  </React.StrictMode>,
  document.getElementById("root")
)
